-------------------------------------------------------------------------------
-- MrCurrency
--
-- Data Broker source for Currency
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

local LibQTip = LibStub("LibQTip-1.0");
MrCurrency = LibStub("AceAddon-3.0"):NewAddon("MrCurrency",
											  "AceEvent-3.0",
											  "AceHook-3.0");

-- Used for clicking the Archaeology currency data source
local ArchaeologySpellId = 78670;	
										
function MrCurrency:OnInitialize()
	-- Create our Data Broker Pieces
	local LDB = LibStub("LibDataBroker-1.1", true);
	if LDB then
		-- Money + Currency Display
		self.dataSource = LDB:NewDataObject("MrCurrency", {
	    	type = "data source",
			text = "Loading...",
			label = "MrCurrency",
			icon = "Interface\\Icons\\INV_Misc_Coin_02",
	    	OnClick = function(frame, button)
	        	-- Do something interesting here
				ToggleCharacter("TokenFrame");
	    	end,
			OnEnter = function(frame)
				-- Show tooltip
				MrCurrency:ShowCurrencyTooltip(frame);
			end,
			OnLeave = function()
				-- Clean up tooltip
			    LibQTip:Release(MrCurrency.activeTooltip);
				MrCurrency.activeTooltip = nil;
		    end,
		});
		
		-- Archaeology Currency Display
		local _, _, archIcon = GetSpellInfo(ArchaeologySpellId);
		self.archaeologySource = LDB:NewDataObject("MrCurrency Artifacts", {
			type = "data source",
			text = "Loading...",
			label = "MrCurrency Artifacts",
			icon = archIcon,
			OnClick = function(frame, button)
				-- Show Archaeology Frame
				local spellName = GetSpellInfo(ArchaeologySpellId);
				if ( spellName ) then CastSpellByName(spellName) end;
			end,
			OnEnter = function(frame)
				-- Show tooltip
				MrCurrency:ShowArtifactTooltip(frame);
			end,
			OnLeave = function()
				-- Clean up tooltip
			    LibQTip:Release(MrCurrency.activeTooltip);
				MrCurrency.activeTooltip = nil;
		    end,
		})
	end
end

function MrCurrency:OnEnable()
	-- Load our DB
	self.Data:Load();

	self:RegisterEvent("PLAYER_MONEY", "OnUpdate");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", "OnUpdate"); -- Also handled by the TokenFrame_Update hook?
	self:RegisterEvent("KNOWN_CURRENCY_TYPES_UPDATE", "OnUpdate");
	self:SecureHook("TokenFrame_LoadUI");	
	
	self:OnUpdate();
end

function MrCurrency:TokenFrame_LoadUI()
	self:Unhook("TokenFrame_LoadUI");
	self:SecureHook("TokenFrame_Update", "OnUpdate");	
end

function MrCurrency:OnUpdate()
	self:UpdateCurrency();
	self:UpdateArtifacts();
end

function MrCurrency:UpdateCurrency()
	local money = GetMoney();
	local text = "";

	local currencies = self.Utils:ReadCurrency();
	
	for _, currency in ipairs(currencies) do
		if( not currency.isUnused and currency.count > 0 ) then
			text = self.Utils:AppendCurrency(text, currency.count, currency.icon, nil, 20);
		end
	end
	
	self.Data:StoreCurrencyInfo(money, currencies);
	
	text = self.Utils:AppendMoneyString(text, money, 12);
	
	self.dataSource.text = text;
end

function MrCurrency:ShowCurrencyTooltip(dataFrame)
	local knownCurrencies = self.Data:GetKnownCurrencyList();
	local currencyTotals = {};
	
	local maxColumns = #knownCurrencies + 2;
	local defaultAlignments = {};
	local line = {};
	
	wipe(line);
	line[1] = "";
	defaultAlignments[1] = "LEFT"
	
	for _, currency in ipairs(knownCurrencies) do
		line[#line+1] = "|T" .. currency.icon .. ":24:24|t";
		defaultAlignments[#defaultAlignments+1] = "RIGHT";
	end
	
	line[#line+1] = "|TInterface\\Icons\\INV_Misc_Coin_02:24:24|t";
	defaultAlignments[#defaultAlignments+1] = "RIGHT";
	
	self.activeTooltip = LibQTip:Acquire("MrCurrencyTooltip", maxColumns, unpack(defaultAlignments));
	self.activeTooltip:SetCellMarginH(4);
	self.activeTooltip:SetCellMarginV(2);
	self.activeTooltip:AddHeader(unpack(line));
	
	for playerName, playerEntry in self.Utils:OrderedPairs(self.Data:GetCurrencyInfo()) do
		local playerLine = {};
		playerLine[1] = playerName;
		
		for _, currency in ipairs(knownCurrencies) do
			playerLine[#playerLine+1] = playerEntry[currency.icon] or 0;
			currencyTotals[currency.icon] = (currencyTotals[currency.icon] or 0) + (playerEntry[currency.icon] or 0);
		end
		
		playerLine[#playerLine+1] = self.Utils:AppendMoneyString(nil, playerEntry["_money"], 12, 2);
		currencyTotals["_money"] = (currencyTotals["_money"] or 0) + (playerEntry["_money"] or 0);
		self.activeTooltip:AddLine(unpack(playerLine));
	end
	
	self.activeTooltip:AddSeparator();
	
	wipe(line);
	line[1] = "Total";
	for _, currency in ipairs(knownCurrencies) do
		line[#line+1] = currencyTotals[currency.icon] or 0;
	end
	line[#line+1] = self.Utils:AppendMoneyString(nil, currencyTotals["_money"] or 0, 12, 2);
	self.activeTooltip:AddLine(unpack(line));
	
	self.activeTooltip:SmartAnchorTo(dataFrame);
	self.activeTooltip:Show();
end

function MrCurrency:UpdateArtifacts()
	local text = "";
	
	local raceCount = GetNumArchaeologyRaces();
	for index=1,raceCount do
		local _, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
		local currencyName, count, currencyIcon, _, _, _ = GetCurrencyInfo(currencyId);
		
		if( count and count > 0 ) then
			text = self.Utils:AppendCurrency(text, count, "Interface\\Icons\\" .. currencyIcon, nil, 20);
		end
	end
	
	if( text == "" ) then text = "No Artifacts" end;
	--if( archIndex == nil ) then self.archaeologySource.icon = ;
	self.archaeologySource.text = text;
end

function MrCurrency:ShowArtifactTooltip(dataFrame)
	self.activeTooltip = LibQTip:Acquire("MrCurrency_ArtifactsTooltip", 2, "LEFT", "RIGHT");
	local _, _, archIndex = GetProfessions();
	
	if(archIndex == nil) then
		self.activeTooltip:AddLine("Visit a trainer to learn archaeology.");
	else
		self.activeTooltip:SetCellMarginH(8);
		self.activeTooltip:SetCellMarginV(2);
		self.activeTooltip:AddHeader("Race", "Tokens");
	
		local raceCount = GetNumArchaeologyRaces();
	
		for index=1,raceCount do
			local raceName, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
			local currencyName, count, currencyIcon, _, _, _ = GetCurrencyInfo(currencyId);
		
			if( count and count > 0 ) then
				self.activeTooltip:AddLine(raceName, count .. " |TInterface\\Icons\\" .. currencyIcon .. ":24:24:0:4|t");
			end
		end
	end
	
	self.activeTooltip:SmartAnchorTo(dataFrame);
	self.activeTooltip:Show();
end